/********************************************************************
"When a Strange Server Calls", 01_general_eval.sql
Author:				Jen McCown (Jen@MidnightDBA.com)
Scripts version:	0.2
Session materials:	http://www.midnightdba.com/Jen/StrangeServer/

See our live weekly webshow "DBAs@Midnight" Fridays 11pm CST 
http://bit.ly/DBAShow

---

Whats on the server  versions, DBs, space

Contents: 
-- Server Information (from xp_msver, sys.dm_os_sys_info, @@servername, @@version)
-- Essential overview: Server info, errors and failures.
	-- Server info: name, versions
	-- Database info: location, size, log size/useage
	-- Errors: SQL Server log and agent log [POWERSHELL]
	-- sp_Blitz;
*********************************************************************/

---------------------------------------------------------------
-- Server Information (from xp_msver, sys.dm_os_sys_info, @@servername, @@version)
---------------------------------------------------------------
CREATE TABLE #SqlServerInfo ( 
	Id int
	, Name varchar(800)
	, InternalValue varchar(800)
	, CharacterValue varchar(800) 
) 

INSERT INTO #SqlServerInfo 
exec xp_msver;

Update #SqlServerInfo SET [Name] = [Name] + ' (Mb)' where Name='PhysicalMemory';
Update #SqlServerInfo SET [Name] = [Name] + ': ';

WITH CTE AS (
	select cast(CAST (sqlserver_start_time as DATE) as varchar(50)) as sqlserver_start_date --*-- 2008+
		,virtual_machine_type_desc
	from sys.dm_os_sys_info
	)
INSERT INTO #SqlServerInfo
select -1 as ID, 'Server Name: ' as [name], NULL as InternalValue, @@servername as CharacterValue
union
select 0 as ID, 'Eval Date: ' as [name], NULL as InternalValue, cast(GetDate() as varchar(500)) as CharacterValue
UNION
SELECT 100 as [id], 'SQL Server start: ' as [name], NULL as InternalValue, sqlserver_start_date as CharacterValue FROM CTE
UNION
SELECT 101 as [id], 'Virtual Machine Type: ' as [name], NULL as InternalValue, virtual_machine_type_desc as CharacterValue FROM CTE
UNION
SELECT 2 AS id, 'ProductVersion [readable]: ' AS [name], NULL AS InternalValue, @@version AS CharacterValue;

select [name], CharacterValue 
from #SqlServerInfo
where id NOT in (5, 6, 9, 10, 11, 12, 13, 14, 17, 18, 20)
order by id, [name];


--  DROP TABLE #SqlServerInfo;


---------------------------------------------------------------
-- Databases: Size and location
---------------------------------------------------------------
SELECT  DB_NAME(database_id) AS Database_Name,
		database_id ,
        file_id ,
        type, type_desc ,
        name ,
        physical_name ,
        state ,
        state_desc ,
        size ,
		size*8/1024 AS SizeInMb,
        max_size ,
        growth ,
		growth * 8 / 1024 AS GrowthInMb, -- growth should really (generally) be by 1 Gb (1024 Mb)...with some exceptions.
        is_media_read_only ,
        is_read_only ,
        is_sparse ,
        is_percent_growth , 
		CASE 
			WHEN is_percent_growth =1 THEN 1
			WHEN type IN (0, 1) and growth%65536 <> 0 THEN 1
			ELSE 0
		END AS Has_Stupid_Growth_Rate
FROM sys.master_files;







--*-- Note: Log files CAN'T be initialized instantly. See http://msdn.microsoft.com/en-us/library/ms175935(v=sql.105).aspx

---------------------------------------------------------------
-- Logs: Space and useage:
---------------------------------------------------------------

DBCC SQLPERF (LOGSPACE);


---------------------------------------------------------------
-- Errors: SQL Server log [POWERSHELL]
---------------------------------------------------------------

-- See http://midnightdba.itbookworm.com/Video/Watch?VideoId=366
-- Filtering: http://colleenmorrow.com/2012/09/20/parsing-windows-event-logs-with-powershell/
-- Also (outfile truncation), http://www.powershellpro.com/what-you-must-know-about-out-file/24/

-- Get all sql server "error" messages from app log, after 3/29/2013, output to file**
--*-- Get-EventLog -ComputerName JENCON -LogName APPLICATION -Source MSSQLSERVER -After 04/10/2013 | Where-Object { $_.Message -like '*error*' } | format-table -Wrap -AutoSize | out-file "c:\temp\Errors.txt" -width 1000; Notepad.exe "c:\temp\Errors.txt"

		-- ** Future: Will go to a SQL table for better hallway vision.

/*
-- Alternate method to play with --
Default instance:

	$a = dir sqlserver:\sql\<servername>
	$a.readerrorlog()

Named instance: 
	$a = dir sqlserver:\sql\<servername> | ?{$_.InstanceName -eq "<instancename>"}
	$a.readerrorlog()

All named instances, with the instance name above each list:
	$a = dir sqlserver:\sql\<servername> 
	$a | %{$_.InstanceName; $_.ReadErrorLog()}

# Also shows us how to get all logs	

*/



/***************************************************************************
*****************************************************************************
sp_Blitz;
****************************************************************************/

SELECT 'Please go get your very own, totally updated version of sp_Blitz from http://www.brentozar.com/blitz/';