/***************************************************************************
*****************************************************************************
"When a Strange Server Calls", 02_perf_eval.sql
Author:				Jen McCown (Jen@MidnightDBA.com)
Scripts version:	0.2
Session materials:	http://www.midnightdba.com/Jen/StrangeServer/

See our live weekly webshow "DBAs@Midnight" Fridays 11pm CST 
http://bit.ly/DBAShow

---

Disk - start at the lowest level and work up into DB level:
	1.	Check partitionOffset, Cluster size.

	see PartitionOffset.ps1

	2.	Ask SAN guys for RAID level. 
		Rule of thumb: Anything over 30% writes should be RAID 10 (not 5); RAID 1 for TEMPDB. 

	3.	Use Disk Defragmenter: Check that data/log drives aren't fragged. (Generally, disk before indexes.)

	------------------------------------------------------
	
	4. Check Data/log file config.
		Data and log should be on different drives.
		Make sure growth rates aren't stupid.
****************************************************************************/
SELECT  DB_NAME(database_id) AS Database_Name,
		database_id ,
        file_id ,
        type, type_desc ,
        name ,
        physical_name ,
        state ,
        state_desc ,
        size ,
		size*8/1024 AS SizeInMb,
        max_size ,
        growth ,
		growth * 8 / 1024 AS GrowthInMb, -- growth should really (generally) be by 1 Gb (1024 Mb)...with some exceptions.
        is_media_read_only ,
        is_read_only ,
        is_sparse ,
        is_percent_growth , 
		CASE 
			WHEN is_percent_growth =1 THEN 1
			WHEN type IN (0, 1) and growth%65536 <> 0 THEN 1
			ELSE 0
		END AS Has_Stupid_Growth_Rate
FROM sys.master_files;


/****************************************************************************
*****************************************************************************
Check indexes 
	5.	Fragmentation - One of the most common problems so it's high on the list.
	6.	Missing.
	7.	Unused.
****************************************************************************/
-----------------------------------------------
-- 5. Fragmentation
	-- WARNING: This may take a significant time if there are a lot of databases, or a lot of indexes (period).
	--			Run on production AT YOUR OWN RISK.

	-- You can either do this: 
	DECLARE @dbid INT ,
		@frag TINYINT = 30 ,	-- Only return indexes where fragmentation in % exceeds this number.
		@pages TINYINT = 100;	-- Only return indexes whose page count meets or exeeds this number.
/*
    SELECT  *
    FROM    sys.dm_db_index_physical_stats(NULL, NULL, NULL, NULL, NULL)
    WHERE   avg_fragmentation_in_percent > @frag
            AND page_count > @pages;
*/
	-- Or, if some databases are closed, or you otherwise want to exclude them, use the following query.
	-- Original query at http://bit.ly/14H74Wb (Indexes from Every Angle, SQLSkills.com)
	-- Modified to run for all online user databases, with customer fragmentation and page # filters:

	DECLARE @result TABLE
		(
		  database_id SMALLINT NULL ,
		  object_id INT NULL ,
		  index_id INT NULL ,
		  partition_number INT NULL ,
		  index_type_desc NVARCHAR(60) NULL ,
		  alloc_unit_type_desc NVARCHAR(60) NULL ,
		  index_depth TINYINT NULL ,
		  index_level TINYINT NULL ,
		  avg_fragmentation_in_percent FLOAT NULL ,
		  fragment_count BIGINT NULL ,
		  avg_fragment_size_in_pages FLOAT NULL ,
		  page_count BIGINT NULL ,
		  avg_page_space_used_in_percent FLOAT NULL ,
		  record_count BIGINT NULL ,
		  ghost_record_count BIGINT NULL ,
		  version_ghost_record_count BIGINT NULL ,
		  min_record_size_in_bytes INT NULL ,
		  max_record_size_in_bytes INT NULL ,
		  avg_record_size_in_bytes FLOAT NULL ,
		  forwarded_record_count BIGINT NULL
		);
	  
	DECLARE AllDBs CURSOR
	FOR
		SELECT  database_id
		FROM    sys.databases
		WHERE   state = 0 -- online
				AND database_id > 4;

	OPEN AllDBs
	FETCH NEXT FROM AllDBs INTO @dbid;

	WHILE @@FETCH_STATUS = 0 
		BEGIN
			INSERT  INTO @result
					SELECT  database_id ,
							object_id ,
							index_id ,
							partition_number ,
							index_type_desc ,
							alloc_unit_type_desc ,
							index_depth ,
							index_level ,
							avg_fragmentation_in_percent ,
							fragment_count ,
							avg_fragment_size_in_pages ,
							page_count ,
							avg_page_space_used_in_percent ,
							record_count ,
							ghost_record_count ,
							version_ghost_record_count ,
							min_record_size_in_bytes ,
							max_record_size_in_bytes ,
							avg_record_size_in_bytes ,
							forwarded_record_count
					FROM    sys.dm_db_index_physical_stats(@dbid, NULL,
														   NULL, NULL, NULL)
					WHERE   avg_fragmentation_in_percent > @frag
							AND page_count > @pages;
			FETCH NEXT FROM AllDBs INTO @DBID;
		END


	CLOSE AllDBs;
	DEALLOCATE ALLDBs;

	SELECT  DB_NAME(database_id) AS Database_Name, *
	FROM    @result
	ORDER BY database_name, avg_fragmentation_in_percent desc;

-----------------------------------------------
-----------------------------------------------
-- [ include scripts in the download ]
-- 6. Missing indexes
	SELECT  D.index_handle
		  , [statement] AS full_object_name
		  , unique_compiles
		  , avg_user_impact
		  , user_scans
		  , user_seeks
		  , column_name
		  , column_usage
		FROM    sys.dm_db_missing_index_groups G
			JOIN sys.dm_db_missing_index_group_stats GS ON G.index_group_handle = GS.group_handle
			JOIN sys.dm_db_missing_index_details D ON G.index_handle = D.index_handle
			CROSS APPLY sys.dm_db_missing_index_columns(D.index_handle) DC
	ORDER BY D.index_handle
		  , [statement] ;

-----------------------------------------------
-----------------------------------------------
-- 7. Low/Unused Indexes
	--  Look into user table indexes that are nonclustered, non-primary key, non-unique, and that are in current database.
	-- Source: http://sqlserverpedia.com/wiki/Find_Indexes_Not_In_Use#T-SQL_Code
	-- [ need sample data ]


    SELECT  o.name ,
            indexname = i.name ,
            i.index_id ,
            reads = user_seeks + user_scans + user_lookups ,
            writes = user_updates ,
            rows = ( SELECT SUM(p.rows)
                     FROM   sys.partitions p
                     WHERE  p.index_id = s.index_id
                            AND s.OBJECT_ID = p.OBJECT_ID
                   ) ,
            CASE WHEN s.user_updates < 1 THEN 100
                 ELSE 1.00 * ( s.user_seeks + s.user_scans + s.user_lookups )
                      / s.user_updates
            END AS reads_per_write ,
            'DROP INDEX ' + QUOTENAME(i.name) + ' ON ' + QUOTENAME(c.name)
            + '.' + QUOTENAME(OBJECT_NAME(s.OBJECT_ID)) AS 'drop statement'
    FROM    sys.dm_db_index_usage_stats s
            INNER JOIN sys.indexes i ON i.index_id = s.index_id
                                        AND s.OBJECT_ID = i.OBJECT_ID
            INNER JOIN sys.objects o ON s.OBJECT_ID = o.OBJECT_ID
            INNER JOIN sys.schemas c ON o.schema_id = c.schema_id
    WHERE   OBJECTPROPERTY(s.OBJECT_ID, 'IsUserTable') = 1
            AND s.database_id = DB_ID()
            AND i.type_desc = 'nonclustered'
            AND i.is_primary_key = 0
            AND i.is_unique_constraint = 0
            AND ( SELECT    SUM(p.rows)
                  FROM      sys.partitions p
                  WHERE     p.index_id = s.index_id
                            AND s.OBJECT_ID = p.OBJECT_ID
                ) > 10000
    ORDER BY reads_per_write;

-----------------------------------
-----------------------------------



	
/****************************************************************************
***************************************************************************
Check memory config
	o	64/32bit
	o	Check for OS/SQL mismatch: if you've got a 32 bit SQL on a 64 bit Windows box, you can't use the extended mem.
	o	if 64 does it have plenty of mem?
	o	if 32 does is have extra mem with AWE turned on?  
	o	Has mem been capped for some reason and starving sql?
	
*/

-- Platform = OS platform; FileDescription = SQL; PhysicalMemory in MB
EXEC xp_msver 'Platform', 'FileDescription','PhysicalMemory';  

SELECT name AS [Is AWE Enabled], value_in_use 
FROM sys.configurations
WHERE name = 'awe enabled';

--EXEC sp_configure 'show advanced options', 1;
--reconfigure

EXEC sp_configure; -- see the "max server memory (MB)" run value. 


/***************************************************************************
***************************************************************************
Check sp_configure values
	o	maxDOP
	o	Adhoc workloads
	o	check other settings to see if they're unusual (look in the sp_Blitz results)
*/


EXEC SP_CONFIGURE 'show advanced options', 1;
GO
RECONFIGURE;
GO
DECLARE @spConfigureOutput TABLE
    (
      [name] VARCHAR(255) ,
      [minimum] INT ,
      [maximum] INT ,
      [config_value] INT ,
      [run_value] INT
    );
 
INSERT  INTO @spConfigureOutput
        EXEC SP_CONFIGURE

SELECT * FROM @spConfigureOutput WHERE name IN ('max degree of parallelism', 'optimize for ad hoc workloads');
GO
EXEC SP_CONFIGURE 'show advanced options', 0;
GO
RECONFIGURE;
GO

/***************************************************************************
***************************************************************************
	Check Trace Flags
	o	check the startup flags
	o	check current flags
*/

-- displays the status of all trace flags that are currently enabled globally.
DBCC TRACESTATUS(-1);

-- For testing: 
-- DBCC TRACEON (3604,-1);
-- DBCC TRACEOFF (3604);

--!-- [Working on a Powershell script to check current startup flags. For now, look manually.]