/***************************************************************************
*****************************************************************************
"When a Strange Server Calls", 03_operational_eval.sql
Author:				Jen McCown (Jen@MidnightDBA.com)
Scripts version:	0.2
Session materials:	http://www.midnightdba.com/Jen/StrangeServer/

See our live weekly webshow "DBAs@Midnight" Fridays 11pm CST 
http://bit.ly/DBAShow

---

Check backups
	1.	Missing? Check recovery models, fill any holes in the backup process.
	2.  Failed?
	3.	Check alerting.  effective alerting, not stormed? Talk to DBAs.
	4.	DBs not backed up to data drive?

	------------------------------------------------------
****************************************************************************/

---------------------------------------------------------------
-- 1. Databases Missing a Data (aka Full) Back-Up Within Past 24 Hours 
--Src: http://www.mssqltips.com/sqlservertip/1601/script-to-retrieve-sql-server-database-backup-history-and-no-backups/
------------------------------------------------------------------------------------------- 
SELECT 'Databases Missing a Data (aka Full) Back-Up Within Past 24 Hours' AS [Note]
UNION SELECT '---------------------------------------------------------------' AS [Note];

SELECT  CONVERT(CHAR(100), SERVERPROPERTY('Servername')) AS Server ,
        msdb.dbo.backupset.database_name ,
        MAX(msdb.dbo.backupset.backup_finish_date) AS last_db_backup_date ,
        DATEDIFF(hh, MAX(msdb.dbo.backupset.backup_finish_date), GETDATE()) AS [Backup Age (Hours)]
FROM    msdb.dbo.backupset
WHERE   msdb.dbo.backupset.type = 'D'
GROUP BY msdb.dbo.backupset.database_name
HAVING  ( MAX(msdb.dbo.backupset.backup_finish_date) < DATEADD(hh, -24,
                                                              GETDATE()) )
UNION  

--Databases without any backup history 
SELECT  CONVERT(CHAR(100), SERVERPROPERTY('Servername')) AS Server ,
        master.dbo.sysdatabases.NAME AS database_name ,
        NULL AS [Last Data Backup Date] ,
        9999 AS [Backup Age (Hours)]
FROM    master.dbo.sysdatabases
        LEFT JOIN msdb.dbo.backupset ON master.dbo.sysdatabases.name = msdb.dbo.backupset.database_name
WHERE   msdb.dbo.backupset.database_name IS NULL
        AND master.dbo.sysdatabases.name <> 'tempdb'
ORDER BY msdb.dbo.backupset.database_name



---------------------------------------------------------------
-- 1a. FULL mode databases Missing log Back-Up Within Past 24 Hours 
------------------------------------------------------------------------------------------- 
SELECT 'FULL mode databases Missing log Back-Up Within Past 24 Hours' AS [Note]
UNION SELECT '---------------------------------------------------------------' AS [Note];

SELECT  CONVERT(CHAR(100), SERVERPROPERTY('Servername')) AS Server ,
        msdb.dbo.backupset.database_name ,
        MAX(msdb.dbo.backupset.backup_finish_date) AS last_db_backup_date ,
        DATEDIFF(hh, MAX(msdb.dbo.backupset.backup_finish_date), GETDATE()) AS [Backup Age (Hours)]
FROM    msdb.dbo.backupset
WHERE   msdb.dbo.backupset.type = 'L'
	AND msdb.dbo.backupset.recovery_model='FULL'
GROUP BY msdb.dbo.backupset.database_name
HAVING  ( MAX(msdb.dbo.backupset.backup_finish_date) < DATEADD(hh, -24,
                                                              GETDATE()) )
UNION  

--Databases without any backup history 
SELECT  CONVERT(CHAR(100), SERVERPROPERTY('Servername')) AS Server ,
        master.dbo.sysdatabases.NAME AS database_name ,
        NULL AS [Last Data Backup Date] ,
        9999 AS [Backup Age (Hours)]
FROM    master.dbo.sysdatabases
        LEFT JOIN msdb.dbo.backupset ON master.dbo.sysdatabases.name = msdb.dbo.backupset.database_name
WHERE   msdb.dbo.backupset.database_name IS NULL
        AND master.dbo.sysdatabases.name <> 'tempdb'
ORDER BY msdb.dbo.backupset.database_name;
	
---------------------------------------------------------------
-- 2. Backup jobs failing? (last 30 days)	
---------------------------------------------------------------
SELECT 'Jobs failing (last 30 days).' AS [Note]
UNION SELECT '---------------------------------------------------------------' AS [Note];

SELECT  j.name AS JobName ,
        jh.step_id ,
        jh.step_name ,
        jh.run_status ,
        jh.sql_message_id ,
        jh.sql_severity ,
        jh.message ,
        jh.run_date ,
        jh.operator_id_emailed, 
		CASE jh.operator_id_emailed 
		WHEN 0 THEN 1 
			ELSE 0
			END AS Nobody_Was_Alerted_To_Failure
FROM    msdb..sysjobhistory jh
        INNER JOIN msdb..sysjobs j ON jh.job_id = j.job_id
WHERE   run_status = 0			-- failed
        AND run_date >= CAST(( CONVERT(VARCHAR(100), GETDATE() - 30, 112) ) AS INT)
ORDER BY run_date DESC ,
        run_time DESC;


---------------------------------------------------------------
--	3.	Check alerting.  Effective alerting, not stormed? Talk to DBAs.
---------------------------------------------------------------


------------------------------------------------------------------------------------------- 
-- 4.	DBs not backed up to data drive? Check by hand. [eventually, compare via sys.master_files]
--Most Recent Database Backup for Each Database - Detailed 
--Src: http://www.mssqltips.com/sqlservertip/1601/script-to-retrieve-sql-server-database-backup-history-and-no-backups/
------------------------------------------------------------------------------------------- 
SELECT 'Most Recent Database Backup for Each Database - Detailed (check that they''re not going to data drive).' AS [Note]
UNION SELECT '---------------------------------------------------------------' AS [Note];


SELECT  A.[Server] ,
        A.last_db_backup_date ,
        B.backup_start_date ,
        B.expiration_date ,
        B.backup_size ,
        B.physical_device_name 
FROM    ( SELECT    CONVERT(CHAR(100), SERVERPROPERTY('Servername')) AS Server ,
                    msdb.dbo.backupset.database_name ,
                    MAX(msdb.dbo.backupset.backup_finish_date) AS last_db_backup_date
          FROM      msdb.dbo.backupmediafamily
                    INNER JOIN msdb.dbo.backupset ON msdb.dbo.backupmediafamily.media_set_id = msdb.dbo.backupset.media_set_id
          WHERE     msdb..backupset.type = 'D'
          GROUP BY  msdb.dbo.backupset.database_name
        ) AS A
        LEFT JOIN ( SELECT  CONVERT(CHAR(100), SERVERPROPERTY('Servername')) AS Server ,
                            msdb.dbo.backupset.database_name ,
                            msdb.dbo.backupset.backup_start_date ,
                            msdb.dbo.backupset.backup_finish_date ,
                            msdb.dbo.backupset.expiration_date ,
                            msdb.dbo.backupset.backup_size ,
                            msdb.dbo.backupmediafamily.physical_device_name ,
                            msdb.dbo.backupset.name AS backupset_name ,
                            msdb.dbo.backupset.description
                    FROM    msdb.dbo.backupmediafamily
                            INNER JOIN msdb.dbo.backupset ON msdb.dbo.backupmediafamily.media_set_id = msdb.dbo.backupset.media_set_id
                    WHERE   msdb..backupset.type = 'D'
                  ) AS B ON A.[server] = B.[server]
                            AND A.[database_name] = B.[database_name]
                            AND A.[last_db_backup_date] = B.[backup_finish_date]
ORDER BY A.database_name;


/***************************************************************************
*****************************************************************************

[manual, and DBAs]

Check index maint
o	Often, maint plans with default options, and checkdb with reindexes. These should be separate.
o	Are reindexes even happening?
o	Are checkdbs even happening?
o	if so to both, then how are they being monitored?
o	Are they at an appropriate interval?
	
****************************************************************************/


/***************************************************************************
*****************************************************************************

Check current traces
o	Traces can bring down a server so let's see if there are lots of them being run all the time.
o	How necessary are they?
****************************************************************************/
SELECT * FROM sys.fn_trace_getinfo(0) ;
-- This should return 5 rows, for traceid=1. This is SQL Server's default trace.

		-- For more info on the default trace, see Simple-Talk: http://bit.ly/ZqOrQE --

/***************************************************************************
*****************************************************************************
	Check other apps -is anything else on the box?
 
****************************************************************************/

